(function () {
    'use strict';

    angular
        .module('cml-adherence-app', [
            'ionic',
            'pascalprecht.translate',
            'ngCordova',
            'ion-affix',
            'n3-line-chart',
            'angular.filter',
            'ngIOS9UIWebViewPatch'
        ])
        .constant('_', window._)
        .constant('appVersion', 1)
        .run(runBlock)
        .config(configBlock);

    function runBlock($rootScope, $window, $ionicPlatform, $log, appVersion, cmlLanguageService, cmlOrganisationService, cmlMedicationsService, cmlLocalNotificationsService, cmlTutorialService, $translate, $cordovaSplashscreen, _) {

        // update routines

        var updateRoutines = [

            {
                version: 1,
                update: function () {
                    var medications = angular.fromJson($window.localStorage['cml.medications']);

                    if (angular.isDefined(medications) && angular.isDefined(medications.medications)) {
                        var date = new Date();
                        date.setMonth(date.getMonth() - 1);

                        _.each(medications.medications, function (medication) {
                            if (angular.isUndefined(medication.createdAt))
                                medication.createdAt = date;
                        });

                        $window.localStorage['cml.medications'] = angular.toJson(medications);
                    }
                }
            }

        ];

        // update if necessary

        var currentAppVersion = $window.localStorage['cml.version'] || 0;

        if (currentAppVersion < appVersion) {
            for (var i = 0; i < updateRoutines.length; i++) {
                if (currentAppVersion < updateRoutines[i].version) {
                    updateRoutines[i].update();
                    currentAppVersion = updateRoutines[i].version;
                }
            }
        }

        $window.localStorage['cml.version'] = currentAppVersion;

        // init app

        cmlLanguageService.init();
        cmlOrganisationService.init();
        cmlTutorialService.init();

        $ionicPlatform.ready(function () {
            $log.debug('Hello cmlAdherenceApp');

            if (window.cordova && window.cordova.plugins.Keyboard) {
                cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
            }

            //if (window.StatusBar) {
            //    StatusBar.styleDefault();
            //}

            cmlLocalNotificationsService.init();
            cmlMedicationsService.init();

            setTimeout(function() {
                $cordovaSplashscreen.hide();
            }, 1000)
        });
    }

    function configBlock($stateProvider, $urlRouterProvider, $translateProvider, $ionicConfigProvider, $logProvider) {

        $ionicConfigProvider.tabs.position('bottom');
        $logProvider.debugEnabled(false);

        /*
         * language settings
         */
        $translateProvider.useStaticFilesLoader({
            prefix: 'translations/',
            suffix: '.json'
        });
        $translateProvider.fallbackLanguage('english');

        /*
         * router settings
         */
        $stateProvider

            // setup an abstract state for the tabs directive
            .state('tab', {
                url: "/tab",
                abstract: true,
                templateUrl: "templates/tabs.html",
                controller: 'cmlAppTabsController'
            })

            // Each tab has its own nav history stack:

            .state('tab.reminder', {
                url: '/reminder',
                views: {
                    'tab-reminder': {
                        templateUrl: 'templates/tab-reminder.html',
                        controller: 'cmlReminderController'
                    }
                }
            })

            .state('tab.adherence', {
                url: '/adherence',
                views: {
                    'tab-adherence': {
                        templateUrl: 'templates/tab-adherence.html',
                        controller: 'cmlAdherenceController'
                    }
                }
            })

            .state('tab.adherence-detail', {
                url: '/adherence/:id',
                views: {
                    'tab-adherence': {
                        templateUrl: 'templates/adherence-detail.html',
                        controller: 'cmlAdherenceDetailController'
                    }
                }
            })

            .state('tab.adherence-medication-detail', {
                url: '/adherence/:id/:date',
                views: {
                    'tab-adherence': {
                        templateUrl: 'templates/adherence-medication-detail.html',
                        controller: 'cmlAdherenceMedicationDetailController'
                    }
                }
            })

            .state('tab.lab-results', {
                url: '/lab-results',
                views: {
                    'tab-lab-results': {
                        templateUrl: 'templates/tab-lab-results.html',
                        controller: 'cmlLabResultsController'
                    }
                }
            })

            .state('tab.organisation', {
                url: '/organisation',
                views: {
                    'tab-organisation': {
                        templateUrl: 'templates/tab-organisation.html',
                        controller: 'cmlOrganisationController'
                    }
                }
            })

            .state('tab.profile', {
                url: '/profile',
                views: {
                    'tab-profile': {
                        templateUrl: 'templates/tab-profile.html',
                        controller: 'cmlProfileController'
                    }
                }
            })

            .state('tab.profile-settings-language', {
                url: '/profile/settings/language',
                views: {
                    'tab-profile': {
                        templateUrl: 'templates/profile-settings-language.html',
                        controller: 'cmlSettingsLanguageController'
                    }
                }
            })

            .state('tab.profile-settings-organisation-1', {
                url: '/profile/settings/organisation',
                views: {
                    'tab-profile': {
                        templateUrl: 'templates/profile-settings-organisation-1.html',
                        controller: 'cmlSettingsOrganisationController'
                    }
                }
            })

            .state('tab.profile-settings-organisation-2', {
                url: '/profile/settings/organisation/:region',
                views: {
                    'tab-profile': {
                        templateUrl: 'templates/profile-settings-organisation-2.html',
                        controller: 'cmlSettingsOrganisationController'
                    }
                }
            })

            .state('tab.profile-medications', {
                url: '/profile/medications',
                views: {
                    'tab-profile': {
                        templateUrl: 'templates/medications.html',
                        controller: 'cmlMedicationsController'
                    }
                }
            })

            .state('tab.profile-medications-wizard', {
                url: '/profile/medications/:id/wizard',
                data: {
                    tabBarHidden: true
                },
                views: {
                    'tab-profile': {
                        templateUrl: 'templates/medications-wizard.html',
                        controller: 'cmlMedicationsWizardController'
                    }
                }
            })

            .state('tab.profile-notepad', {
                url: '/profile/notepad',
                views: {
                    'tab-profile': {
                        templateUrl: 'templates/notepad.html',
                        controller: 'cmlNotepadController'
                    }
                }
            })

            .state('tab.profile-imprint', {
                url: '/profile/imprint',
                views: {
                    'tab-profile': {
                        templateUrl: 'templates/profile-imprint.html',
                        controller: 'cmlImprintController'
                    }
                }
            })
        ;

        // if none of the above states are matched, use this as the fallback
        $urlRouterProvider.otherwise('/tab/reminder');
    }
})();


