(function () {
    'use strict';

    angular
        .module('cml-adherence-app')
        .controller('cmlAdherenceController', cmlAdherenceController);

    function cmlAdherenceController($scope, $log, cmlMedicationsService, cmlAdherenceService, _) {
        $log.debug('Hello cmlAdherenceController');

        $scope.medications = cmlMedicationsService.getAll();
        $scope.adherence = {};

        function refresh() {
            $scope.adherence = {};

            _.each($scope.medications, function (medication) {
                $scope.adherence[medication.id] = calcAdherence(medication);
            });
        }

        function calcAdherence(medication) {
            var currentDate = new Date();

            var createdSinceDays = Math.ceil(Math.abs(currentDate.getTime() - (medication.createdAt).getTime()) / (1000*60*60*24));
            if (createdSinceDays > 30) createdSinceDays = 30;

            var taken = 0;
            var maxTaken = 0;

            if (medication.interval === 'day') {
                _.each(medication.doses, function () {
                    maxTaken += createdSinceDays;
                });
            } else {
                var date1 = new Date();
                for (var i = 0; i < createdSinceDays; i++) {
                    _.each(medication.doses, function (dose) {
                        if (date1.getDay() == dose.weekDay) {
                            maxTaken++;
                        }
                    });
                    date1.setDate(date1.getDate() - 1);
                }
            }

            var date2 = new Date();
            for (var i = 0; i < createdSinceDays; i++) {
                taken += cmlAdherenceService.getTakenDoses(date2, medication.id).length;
                date2.setDate(date2.getDate() - 1);
            }

            return taken * 100 / maxTaken;
        }


        $scope.$on('$ionicView.afterEnter', function () {
            refresh();
        });
    }
})();
