(function () {
    'use strict';

    angular
        .module('cml-adherence-app')
        .controller('cmlAdherenceDetailController', cmlAdherenceDetailController);

    function cmlAdherenceDetailController($scope, $stateParams, $filter, $log, cmlMedicationsService, cmlAdherenceService, _) {
        $log.debug('Hello cmlAdherenceDetailController');

        var _id = $stateParams.id;
        $scope.medication = cmlMedicationsService.get(parseInt(_id));
        $scope.months = [];

        function refresh() {
            var date = new Date();
            var days = [];

            var createdSinceDays = Math.ceil(Math.abs(date.getTime() - ($scope.medication.createdAt).getTime()) / (1000*60*60*24));
            if (createdSinceDays > 30) createdSinceDays = 30;

            $log.debug('date', date.getTime());
            $log.debug('createdAt', $scope.medication.createdAt.getTime());
            $log.debug('createdSinceDays', createdSinceDays);

            for (var i = 0; i < createdSinceDays; i++) {
                var takenDoses = cmlAdherenceService.getTakenDoses(date, _id);
                var maxDoses = 0;

                if ($scope.medication.interval === 'day') {
                    maxDoses = $scope.medication.doses.length;
                } else {
                    var weekDay = date.getDay();
                    _.forEach($scope.medication.doses, function (dose) {
                        if (dose.weekDay == weekDay) {
                            maxDoses++;
                        }
                    });
                }

                var doses = [];
                for (var d = 0; d < takenDoses.length; d++) {
                    doses.push({status: 'taken'});
                }
                for (var d = 0; d < (maxDoses - takenDoses.length); d++) {
                    doses.push({status: 'untaken'});
                }

                if (maxDoses > 0) {
                    days.push({
                        date: new Date(date),
                        month: date.getMonth(),
                        day: date.getDate(),
                        takenDoses: takenDoses,
                        maxDoses: maxDoses,
                        doses: doses
                    });
                }

                date.setDate(date.getDate() - 1);
            }

            $scope.months = renderData(days);
        }

        function renderData(days) {
            var monthsGrouped = $filter('groupBy')(days, 'month');
            var months = [];

            var k = Object.keys(monthsGrouped);
            _.each(k, function (month) {
                months.push({
                    date: new Date(0, month),
                    month: month,
                    days: monthsGrouped[month]
                })
            });

            return months;
        }

        $scope.$on('$ionicView.afterEnter', function () {
            refresh();
        });
    }
})();
