(function () {
    'use strict';

    angular
        .module('cml-adherence-app')
        .controller('cmlLabResultsController', cmlLabResultsController);

    function cmlLabResultsController($scope, $filter, $log, cmlLabResultsService, cmlLabResultValues, $ionicModal, _, $translate, $cordovaDialogs) {
        $log.debug('Hello cmlLabResultsController');

        $scope.chartOptions = {
            axes: {
                x: {
                    type: 'linear',
                    ticksRotate: 90,
                    labelFunction: function (v) {

                        if (v == parseInt(v)) {
                            //$log.debug($scope.chartData, v, parseInt(v), $scope.chartData[parseInt(v)], new Date($scope.chartData[parseInt(v)].date));

                            if (angular.isDefined($scope.chartData[parseInt(v)])) {
                                $log.debug('date', $scope.chartData[parseInt(v)].date);
                                var date = new Date($scope.chartData[parseInt(v)].date);
                                return $filter('date')(date, $translate.instant('LAB_RESULTS-DATE-FORMAT-CHART'));
                            }
                        }

                        return '';
                    }
                },
                y: {
                    type: 'log',
                    ticks: [0.001, 0.0032, 0.01, 0.1, 1, 10, 100],
                    //ticksFormat: '.f',
                    ticksFormatter: function (v) {
                        var labels = {
                            '0.001': 'MR5=0.001%',
                            '0.0032': 'MR4.5=0.0032%',
                            '0.01':  'MR4=0.01%',
                            '0.1':   'MMR=0.1%',
                            '1': '1%',
                            '10': '10%',
                            '100': '100%'
                        };

                        // TODO: display format
                        return labels[v];
                    }
                }
            },
            series: [
                {y: 'yVal', color: '#32858d', dotSize: 4}
            ],
            margin: {left: 100, bottom: 75, top: 20, right: 20},
            lineMode: 'linear',
            drawLegend: false
        };
        $scope.chartData = [];
        $scope.chartPointer = 0;
        $scope.chartSize = 10;

        $scope.values = cmlLabResultValues;
        $scope.filter = $scope.values[0];
        $scope.results = cmlLabResultsService.getAll();
        $scope.modal = {};
        $scope.error = false;
        $scope.error_field = '';

        function updateChart() {
            $scope.chartData = [];

            $log.debug($scope.chartPointer, $scope.chartSize, $scope.results.length);

            var chartData = [];
            var data = $filter('orderBy')($scope.results, '-date').splice($scope.chartPointer, $scope.chartSize).reverse();

            _.each(data, function (entry) {
                chartData.push({
                    date: new Date(entry.date),
                    x: chartData.length,
                    yVal: entry['bcr-abl']
                });
            });

            $scope.chartData = chartData;
        }

        $scope.chartLeft = function () {
            if ($scope.chartPointer + $scope.chartSize < $scope.results.length && $scope.chartSize < $scope.results.length) {
                $scope.chartPointer += 1;
                updateChart();
            }
        };
        $scope.chartRight = function () {
            if ($scope.chartPointer > 0) {
                $scope.chartPointer -= 1;
                updateChart();
            }
        };

        updateChart();

        function validateNumber(strNumber) {
            strNumber = strNumber.replace('%', '');
            var number = parseFloat(strNumber.replace(',', '.'));
            return !isNaN(number) && isFinite(number) ? number : false;
        }

        function checkInput(inputField) {
            var inputNumber;

            if (angular.isDefined($scope.modal.data[inputField]) && ($scope.modal.data[inputField] == 0 || !_.isEmpty($scope.modal.data[inputField]))) {
                if ((inputNumber = validateNumber($scope.modal.data[inputField])) === false) {
                    return false;
                }

                $scope.modal.data[inputField] = inputNumber;
            }

            return true;
        }

        $scope.isValue = function (val) {
            return angular.isDefined(val);
        };

        $scope.modalAdd = function () {
            createModal('new', {
                date: new Date()
            });
        };

        $scope.modalEdit = function (result) {
            createModal('edit', _.clone(result));
        };

        $scope.modalSave = function () {
            $scope.error = false;

            // check input data
            var bcrAbl = checkInput('bcr-abl');
            var cytogenetics = checkInput('cytogenetics');
            var hbValue = checkInput('hb-value');
            var leucocytes = checkInput('leucocytes');

            if (bcrAbl && cytogenetics && hbValue && leucocytes) {
                $scope.modal.view.hide();

                if ($scope.modal.mode === 'new') {
                    cmlLabResultsService.add($scope.modal.data);
                } else {
                    cmlLabResultsService.update($scope.modal.data);
                }

                updateChart();
            } else {
                $scope.error = true;

                if (!bcrAbl) {
                    $scope.error_field = 'bcr-abl';
                } else if (!leucocytes) {
                    $scope.error_field = 'leucocytes';
                } else if (!cytogenetics) {
                    $scope.error_field = 'cytogenetics';
                } else if (!hbValue) {
                    $scope.error_field = 'hb-value';
                }

                $cordovaDialogs.alert(
                    $translate.instant('LAB_RESULTS-POPUP-INPUT-ERROR'),
                    $translate.instant('LAB_RESULTS-POPUP-INPUT-ERROR-TITLE')
                );
            }
        };

        $scope.modalRemove = function (id) {
            $cordovaDialogs.confirm(
                $translate.instant('LAB_RESULTS-POPUP-REMOVE-TEMPLATE'),
                $translate.instant('LAB_RESULTS-POPUP-REMOVE-TITLE')
            ).then(
                function (btnIndex) {
                    if (btnIndex === 1) {
                        cmlLabResultsService.remove(id);
                        $scope.modal.view.hide();
                    }

                    updateChart();
                }
            );
        };

        $scope.changeFilter = function (newFilter) {
            $scope.filter = newFilter;
        };

        $scope.getPlaceholder = function (langKey) {
            return (angular.isDefined(langKey) ? langKey : 'LAB_RESULTS-MODAL-PLACEHOLDER-VALUE');
        };

        $scope.getDate = function (result) {
            return $filter('date')(result.date, $translate.instant('LAB_RESULTS-DATE-FORMAT-LIST'));
        };

        var createModal = function (mode, data) {
            if (angular.isDefined($scope.modal.view)) {
                $scope.modal.view.remove();
            }

            if (angular.isUndefined(data)) {
                data = {};
            }

            $ionicModal.fromTemplateUrl('templates/tab-lab-results_modal.html', {
                scope: $scope
                , animation: 'slide-in-up'
            }).then(function (modal) {
                $scope.modal = {
                    mode: mode
                    , view: modal
                    , data: data
                };
                modal.show();
            });
        };
    }
})();
