(function () {
    'use strict';

    angular
        .module('cml-adherence-app')
        .controller('cmlMedicationsWizardController', cmlMedicationsWizardController);

    function cmlMedicationsWizardController($scope, $stateParams, $log, cmlMedicationTemplates, cmlMedicationUnits, cmlAdherenceService, cmlMedicationsService, cmlLocalNotificationsService, $ionicScrollDelegate, $ionicHistory, $translate, $cordovaDialogs) {

        $log.debug('Hello cmlMedicationsWizardController');

        $scope.wizardStep = 1;
        $scope.isNew = $stateParams.id === 'new';
        $scope.selected = {
            unit: {},
            template: {}
        };

        // has permission to receive local notifications
        $scope.hasPermission = false;
        checkPermission();

        $scope.templates = cmlMedicationTemplates;
        $scope.units = cmlMedicationUnits;

        // load medication
        $scope.medication = {};
        if ($stateParams.id !== 'new') {
            $scope.medication = cmlMedicationsService.get($stateParams.id);

            $scope.selected = {
                template: _.find($scope.templates, 'name', $scope.medication.template),
                unit: _.find($scope.units, 'name', $scope.medication.unit)
            };
        }

        function checkInputs() {
            switch ($scope.wizardStep) {
                case 1:
                    return angular.isDefined($scope.medication.template);

                case 2:
                    return angular.isDefined($scope.medication.title)
                        && !_.isEmpty($scope.medication.title)
                        && angular.isDefined($scope.medication.unit)
                        && !_.isEmpty($scope.medication.unit)
                        && angular.isDefined($scope.medication.amount)
                        && _.isNumber($scope.medication.amount)
                        && $scope.medication.amount > 0;

                case 3:
                    return angular.isDefined($scope.medication.interval)
                        && !_.isEmpty($scope.medication.interval);


                case 4:
                    if (!angular.isDefined($scope.medication.doses) || _.isEmpty($scope.medication.doses)) {
                        return false;
                    }

                    var emptyTime = true;
                    _.each($scope.medication.doses, function (dose) {
                        return !(emptyTime = (!angular.isDefined(dose.time) || dose.time === null));
                    });

                    return !emptyTime;

                case 5:
                    return true;
            }

            return false;
        }

        function checkPermission() {
            cmlLocalNotificationsService.checkPermission().then(function (permission) {
                $scope.hasPermission = permission;
            });
        }

        $scope.changeTemplate = function (template) {
            $scope.selected.template = template;

            $scope.medication = {
                title: template.name !== 'other' ? $translate.instant(template.langKey) : '',
                template: template.name,
                interval: template['template'].interval,
                amount: template['template'].amount,
                reminder: false,
                unit: template['template'].unit
            };

            for (var i = 0; i < template['template'].doses; i++) {
                $scope.addDose();
            }

            $scope.selected.unit = _.find($scope.units, {name: template['template'].unit});
        };

        $scope.changeWeekDay = function (dose) {
            var time = new Date(dose.time);
            var weekDay = parseInt(dose['weekDay']);

            time.setDate(time.getDate() + weekDay - time.getDay());
            dose.time = time;
            dose.weekDay = weekDay;
        };

        $scope.addDose = function () {
            if (angular.isUndefined($scope.medication.doses)) {
                $scope.medication.doses = [];
            }

            var time = new Date();
            time.setHours(8, 0, 0, 0);

            var weekDay = (new Date()).getDay();

            $scope.medication.doses.push({
                time: time,
                weekDay: weekDay
            });
        };

        $scope.removeDose = function (index) {
            $cordovaDialogs.confirm(
                $translate.instant('MEDICATIONS-WIZARD-POPUP-4-TEMPLATE'),
                $translate.instant('MEDICATIONS-WIZARD-POPUP-4-TITLE')
            ).then(
                function (btnIndex) {
                    if (btnIndex === 1) {
                        cmlAdherenceService.removeDose($scope.medication, $scope.medication.doses[index].id);
                        $scope.medication.doses.splice(index, 1);
                    }
                }
            );
        };

        $scope.cancel = function () {
            $cordovaDialogs.confirm(
                $translate.instant('MEDICATIONS-WIZARD-POPUP-2-TEMPLATE'),
                $translate.instant('MEDICATIONS-WIZARD-POPUP-2-TITLE')
            ).then(
                function (btnIndex) {
                    if (btnIndex === 1) {
                        $ionicHistory.goBack();
                    }
                }
            );
        };

        $scope.removeMedication = function (index) {
            $cordovaDialogs.confirm(
                $translate.instant('MEDICATIONS-WIZARD-POPUP-3-TEMPLATE'),
                $translate.instant('MEDICATIONS-WIZARD-POPUP-3-TITLE')
            ).then(
                function (btnIndex) {
                    if (btnIndex === 1) {
                        cmlAdherenceService.removeAllDoses($scope.medication);
                        cmlMedicationsService.remove($scope.medication.id);

                        $ionicHistory.goBack();
                    }
                }
            );
        };

        $scope.next = function () {
            $log.debug($scope.medication);

            if (checkInputs()) {
                if ($scope.wizardStep < 5) {
                    $ionicScrollDelegate.scrollTop();
                    $scope.wizardStep++;
                } else {
                    if ($stateParams.id === 'new') {
                        cmlMedicationsService.add($scope.medication);
                    } else {
                        $scope.medication.id = parseInt($stateParams.id);
                        cmlMedicationsService.update($scope.medication);
                    }

                    $ionicHistory.goBack();
                }
            }
        };

        $scope.prev = function () {
            if ($scope.wizardStep > 1) {
                $scope.wizardStep--;
            }
        };

        $scope.$watch('selected.unit', function (newVal) {
            if (newVal) {
                $scope.medication.unit = newVal.name;
            }
        });

        // check permission when app comes from background
        document.addEventListener("resume", function () {
            checkPermission();
        }, false);
    }
})();
