(function () {
    'use strict';

    angular
        .module('cml-adherence-app')
        .controller('cmlReminderController', cmlReminderController);

    function cmlReminderController($scope, $rootScope, $timeout, $log, $state, cmlMedicationsService, cmlAdherenceService, cmlTutorialService, $ionicLoading, $ionicModal, _, $translate, $cordovaDialogs, $ionicScrollDelegate) {
        $log.debug('Hello cmlReminderController');

        $scope.today = new Date();
        $scope.todayDoses = [];
        $scope.dateFormat = $translate.instant('TAB-REMINDER-DATE-FORMAT');
        $scope.timeFormat = $translate.instant('TAB-REMINDER-TIME-FORMAT');

        $scope.initialised = false;

        $scope.tutorial = {
            page: 1,
            pages: 6
        };

        // Tutorial Modal
        $ionicModal.fromTemplateUrl('templates/tutorial_modal.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function (modal) {
            $scope.modalTutorial = modal;
            if (!cmlTutorialService.isDone()) {
                $scope.openTutorial();
            }
        });

        $scope.openTutorial = function () {
            $scope.modalTutorial.show();
        };

        $scope.closeTutorial = function (force, dontShowAgain) {
            if (force) {
                $scope.modalTutorial.hide();

                if (!angular.isDefined(dontShowAgain)) dontShowAgain = false;
                cmlTutorialService.setDone(dontShowAgain);

                if (!dontShowAgain) {
                    // navigate to profile
                    $state.go('tab.profile');
                }
            } else {
                $cordovaDialogs.confirm(
                    $translate.instant('TUTORIAL-SKIP-TEXT'),
                    $translate.instant('TUTORIAL-SKIP-TITLE')
                )
                    .then(function (buttonIndex) {
                        if (buttonIndex === 1) {
                            $scope.modalTutorial.hide();
                            cmlTutorialService.setDone(true);
                        }
                    });
            }

            $scope.tutorial.page = 1;
        };

        $scope.tutorialNext = function () {
            $ionicScrollDelegate.scrollTop();
            if ($scope.tutorial.page + 1 <= $scope.tutorial.pages) {
                $scope.tutorial.page++;
            } else {
                $scope.closeTutorial(true, false);
            }
        };

        $scope.tutorialPrev = function () {
            $ionicScrollDelegate.scrollTop();
            if ($scope.tutorial.page - 1 > 0) {
                $scope.tutorial.page--;
            }
        };

        // Information Modal
        $ionicModal.fromTemplateUrl('templates/cml-information_modal.html', {
            scope: $scope,
            animation: 'slide-in-up'
        }).then(function (modal) {
            $scope.modalInformation = modal;
        });

        $scope.openInformation = function () {
            $scope.modalInformation.show();
        };

        $scope.closeInformation = function () {
            $scope.modalInformation.hide();
        };

        $scope.$on('$destroy', function () {
            $scope.modalTutorial.remove();
            $scope.modalInformation.remove();
        });


        $ionicLoading.show();

        $timeout(function () {
            reload();
            $scope.initialised = true;
            $ionicLoading.hide();
        }, 1000);

        function reload() {
            $scope.today = new Date();
            $scope.todayDoses = [];

            $scope.dateFormat = $translate.instant('TAB-REMINDER-DATE-FORMAT');
            $scope.timeFormat = $translate.instant('TAB-REMINDER-TIME-FORMAT');

            var medications = cmlMedicationsService.getAll();
            var todayDateString = $scope.today.toDateString();

            _.forEach(medications, function (medication) {
                _.forEach(medication.doses, function (dose) {
                    if (todayDateString === dose.time.toDateString() || medication.interval === 'day') {
                        var todayDose = {
                            taken: cmlAdherenceService.hasTakenDose($scope.today, medication.id, dose.id)
                        };

                        $scope.todayDoses.push(_.assign(todayDose, dose, {medication: medication}));
                    }
                });
            });
        }

        $scope.changeStatus = function (dose) {
            if (dose.taken) {
                $log.debug('changeStatus taken:', true);
                $scope.showMedicationTakenInfo(dose);
                cmlAdherenceService.addTakenDose($scope.today, dose.medication, dose.id);
            } else {
                $log.debug('changeStatus taken:', false);
                $scope.medicationTakenInfo = false;
                $timeout.cancel(dose.timeout);
                dose.timeout = null;
                cmlAdherenceService.removeTakenDose($scope.today, dose.medication, dose.id);
            }
        };

        $scope.showMedicationTakenInfo = function (dose) {
            $scope.medicationTakenInfo = true;

            dose.timeout = $timeout(function () {
                $scope.medicationTakenInfo = false;
                $timeout.cancel(dose.timeout);
            }, 2000);
        };

        $scope.getStatus = function (medicationId, doseId) {
            return cmlAdherenceService.hasTakenDose($scope.today, medicationId, doseId);
        };

        $scope.$on('$ionicView.afterEnter', function () {
            if ($scope.initialised) {
                reload();
            }

            if (angular.isDefined($scope.modalTutorial) && !cmlTutorialService.isDone()) {
                $scope.openTutorial();
            }
        });

        $rootScope.$on('cmlMedicationsInit', function () {
            reload();
        });

        document.addEventListener("resume", function () {
            reload();

            // force ui to apply data changes
            $scope.$apply();
        }, false);
    }
})();
