(function () {
    'use strict';

    angular
        .module('cml-adherence-app')
        .factory('cmlAdherenceService', cmlAdherenceService);

    function cmlAdherenceService($log, cmlLocalStorageService, _) {
        $log.debug('Hello cmlAdherenceService');

        var STORAGE_KEY = 'cml.adherence';
        var self = this;

        var _adherence = {};

        function serialize() {
            var _storageData = {
                adherence: _adherence
            };
            cmlLocalStorageService.setObject(STORAGE_KEY, _storageData);
        }

        function deserialize() {
            var _storage = cmlLocalStorageService.getObject(STORAGE_KEY);

            if (angular.isDefined(_storage)) {
                _adherence = _storage.adherence;
            }
        }

        self.init = function () {
            deserialize();
        };

        self.getTakenDoses = function (date, medicationId) {
            var day = new Date(date);
            day.setHours(0, 0, 0, 0);

            var timestamp = day.getTime();

            if (angular.isDefined(_adherence[timestamp]) && angular.isDefined(_adherence[timestamp][medicationId])) {
                return _adherence[timestamp][medicationId];
            } else {
                return [];
            }
        };

        self.hasTakenDose = function (date, medicationId, doseId) {
            var day = new Date(date);
            day.setHours(0, 0, 0, 0);

            var timestamp = day.getTime();

            if (angular.isDefined(_adherence[timestamp]) && angular.isDefined(_adherence[timestamp][medicationId])) {
                return angular.isDefined(_.find(_adherence[timestamp][medicationId], 'id', doseId));
            }

            return false;
        };

        self.addTakenDose = function (date, medication, doseId) {
            var day = new Date(date);
            day.setHours(0, 0, 0, 0);

            var timestamp = day.getTime();

            if (angular.isUndefined(_adherence[timestamp])) {
                _adherence[timestamp] = {};
            }

            if (angular.isUndefined(_adherence[timestamp][medication.id])) {
                _adherence[timestamp][medication.id] = [];
            }

            _adherence[timestamp][medication.id].push({
                id: doseId,
                doses: medication.doses.length
            });

            serialize();
        };

        self.removeTakenDose = function (date, medication, doseId) {
            var day = new Date(date);
            day.setHours(0, 0, 0, 0);

            var timestamp = day.getTime();

            if (angular.isDefined(_adherence[timestamp]) && _adherence[timestamp][medication.id]) {
                var doseIndex = _.findIndex(_adherence[timestamp][medication.id], 'id', doseId);
                $log.debug('removeTakenDose', doseIndex);
                if (doseIndex >= 0) {
                    _adherence[timestamp][medication.id].splice(doseIndex, 1);
                }
            }

            serialize();
        };

        self.removeDose = function (medication, doseId) {
            _.forEach(_adherence, function (adherence) {
                if (angular.isDefined(adherence[medication.id])) {
                    var doseIndex = _.findIndex(adherence[medication.id], 'id', doseId);
                    if (doseIndex > -1) {
                        adherence[medication.id].splice(doseIndex, 1);
                    }
                }
            });

            serialize();
        };

        self.removeAllDoses = function (medication) {
            _.forEach(_adherence, function (adherence) {
                if (angular.isDefined(adherence[medication.id])) {
                    adherence[medication.id] = [];
                }
            });

            serialize();
        };

        self.init();

        return self;
    }
})();
