(function () {
    'use strict';

    angular
        .module('cml-adherence-app')
        .factory('cmlLabResultsService', cmlLabResultsService);

    function cmlLabResultsService($log, cmlLocalStorageService, _) {
        $log.debug('Hello cmlLabResultsService');

        var STORAGE_KEY = 'cml.labResults';
        var self = this;

        var _id = 0;
        var _results = [];

        function getNewId() {
            return (++_id).toString();
        }

        function serialize() {
            var _storageData = {
                lastIndex: _id,
                results: _results
            };
            cmlLocalStorageService.setObject(STORAGE_KEY, _storageData);
        }

        function deserialize() {
            var _storage = cmlLocalStorageService.getObject(STORAGE_KEY);

            if (angular.isDefined(_storage)) {
                _id = _storage.lastIndex;
                _results = _storage.results;

                _.forEach(_results, function (labResult) {
                    labResult.date = cmlLocalStorageService.strToDate(labResult.date);
                });
            }
        }

        self.init = function () {
            deserialize();
        };

        self.getAll = function () {
            return _results;
        };

        self.add = function (result) {
            result.id = getNewId();

            _results.push(result);
            serialize();
        };

        self.update = function (editEntry) {
            var index = _.findIndex(_results, function (entry) {
                return entry.id === editEntry.id;
            });

            if (index !== -1) {
                _results[index] = editEntry;
            }

            serialize();
        };

        self.remove = function (id) {
            var index = _.findIndex(_results, function (entry) {
                return entry.id === id;
            });

            if (index !== -1) {
                _results.splice(index, 1);
            }

            serialize();
        };

        self.init();

        return self;
    }
})();
