(function () {
    'use strict';

    angular.module('cml-adherence-app')
        .factory('cmlLanguageService', cmlLanguageService);

    function cmlLanguageService($log, cmlLocalStorageService, cmlLanguages, $translate) {
        $log.debug('Hello cmlLanguageService');

        var STORAGE_KEY = 'cml.language';

        var self = this;
        var _language;

        function serialize() {
            cmlLocalStorageService.set(STORAGE_KEY, _language);
            $translate.use(_language);
        }

        function deserialize() {
            _language = cmlLocalStorageService.get(STORAGE_KEY);

            if (angular.isUndefined(_language)) {
                _language = tryToGetSysLang();
                serialize();
            }

            $translate.use(_language);
        }

        function tryToGetSysLang() {
            var browserLang = navigator.language || navigator.userLanguage;
            var language = 'english';

            if (browserLang.indexOf('de') === 0) {
                language = 'german';
            } else if (browserLang.indexOf('fr') === 0) {
                language = 'french';
            } else if (browserLang.indexOf('es') === 0) {
                language = 'spanish';
            } else if (browserLang.indexOf('pt') === 0) {
                language = 'portuguese';
            } else if (browserLang.indexOf('he') === 0) {
                language = 'hebrew';
            } else if (browserLang.indexOf('ar') === 0) {
                language = 'arabic';
            }

            return language;
        }

        self.init = function () {
            deserialize();
        };

        self.set = function (language) {
            _language = language;
            serialize();
        };

        self.get = function () {
            deserialize();
            return _language;
        };

        self.isRtl = function () {
            return cmlLanguages[_language].rtl;
        };

        return self;
    }
})();
