(function () {
    'use strict';

    angular
        .module('cml-adherence-app')
        .factory('cmlLocalNotificationsService', cmlLocalNotificationsService);

    function cmlLocalNotificationsService($window, $state, $q, $log, $ionicPopup, _, $cordovaDialogs, $translate) {
        $log.debug('Hello cmlLocalNotificationsService');

        var localNotification;
        var self = this;

        self.init = function () {
            localNotification = $window.plugin.notification.local;

            localNotification.registerPermission(function (granted) {
                $log.debug('Permission has been granted: ' + granted);
            });

            localNotification.on('trigger', function (notification, state) {
                $log.debug('triggered', JSON.stringify(notification), state);

                $cordovaDialogs.alert(
                    notification.data,
                    $translate.instant('MEDICATION-IN-APP-NOTIFICATION-TITLE')
                ).then(
                    function () {
                        $state.go('tab.reminder');
                    }
                );
            }, this);

            localNotification.on('click', function (notification, state) {
                $log.debug('clicked', JSON.stringify(notification), state);
                $state.go('tab.reminder');
            }, this);
        };

        self.checkPermission = function () {
            var d = $q.defer();

            localNotification.hasPermission(function (allowed) {
                d.resolve(allowed);
            });

            return d.promise;
        };

        self.createNotification = function (id, text, altText, at, every) {
            return {
                id: id,
                //title: title,
                text: text,
                at: at,
                every: every,
                data: altText
            };
        };

        self.addNotifications = function (notifications) {
            $log.debug('addNotifications', JSON.stringify(notifications));

            if (!_.isEmpty(notifications)) {
                localNotification.schedule(notifications, function () {
                    $log.debug('localNotification.schedule');
                });
            }
        };

        self.updateNotifications = function (notifications) {
            $log.debug('updateNotifications', JSON.stringify(notifications));

            if (!_.isEmpty(notifications)) {
                localNotification.update(notifications, function () {
                    $log.debug('localNotification.update');
                });
            }
        };

        self.removeNotifications = function (ids) {
            $log.debug('removeNotifications', JSON.stringify(ids));

            if (!_.isEmpty(ids)) {
                localNotification.cancel(ids, function () {
                    $log.debug('localNotification.cancel');
                });
            }
        };

        self.clearNotifications = function (ids) {
            $log.debug('clearNotifications', JSON.stringify(ids));

            if (!_.isEmpty(ids)) {
                localNotification.cancel(ids, function () {
                    $log.debug('localNotification.clear');
                });
            }
        };

        self.getAllNotifications = function () {
            var d = $q.defer();

            localNotification.getAllIds(function (ids) {
                d.resolve(ids);
            });

            return d.promise;
        };

        return self;
    }
})();
