(function () {
    'use strict';

    angular.module('cml-adherence-app')
        .factory('cmlOrganisationService', cmlOrganisationService);

    function cmlOrganisationService($log, $http, $q, cmlLocalStorageService, cmlApiEndpoints) {
        $log.debug('Hello cmlOrganisationService');

        var STORAGE_KEY_ORGANISATIONS = 'cml.organisations';
        var STORAGE_KEY_USER_ORGANISATION = 'cml.organisation';

        var self = this;
        var _organisations;
        var _userOrganisation;

        function serializeOrganisations() {
            cmlLocalStorageService.setObject(STORAGE_KEY_ORGANISATIONS, _organisations);
        }

        function deserializeOrganisations() {
            _organisations = cmlLocalStorageService.getObject(STORAGE_KEY_ORGANISATIONS);

            if (angular.isUndefined(_organisations)) {
                reloadOrganisations();
            }
        }

        function serializeUserOrganisation() {
            cmlLocalStorageService.setObject(STORAGE_KEY_USER_ORGANISATION, _userOrganisation);
        }

        function deserializeUserOrganisation() {
            _userOrganisation = cmlLocalStorageService.getObject(STORAGE_KEY_USER_ORGANISATION);
        }

        function reloadOrganisations() {
            var deferred = $q.defer();

            $http({
                method: 'GET',
                url: cmlApiEndpoints.organisations.url
            }).then(function (organisations) {
                _organisations = organisations.data;
                serializeOrganisations();
                deferred.resolve();
            }, function () {
                deferred.reject();
            });

            return deferred.promise;
        }

        self.init = function () {
            deserializeOrganisations();
            deserializeUserOrganisation();
        };

        self.reload = function () {
            return reloadOrganisations();
        };

        self.getAll = function () {
            return _organisations;
        };

        self.set = function (organisation) {
            _userOrganisation = organisation;
            serializeUserOrganisation();
        };

        self.setContent = function (content) {
            _userOrganisation.content = content;
            serializeUserOrganisation();
        };

        self.get = function () {
            return _userOrganisation;
        };

        return self;
    }
})();
